/*
There are three varieties of inheritance:
1. Public
2. Protected
3. Private
Public is the most common and models the "is-a"
relationship, or "is-a-kind-of".

Contrast this with the "has-a" relationship.

Also contrast with the "is-like-a" relationship.
Inheritance can add properties to a base class, but
should not remove properties from the base class.

Pulbic inheritance does not model an
"is-implemented-as-a" relationship.
For example you could implement a stack by using an array.
However, it isn't proper to derive a Stack class from
an array class. We hide the array properties by giving
the Stack a private array member.

Public inheritance also doesn't model a "uses-a" relation.
Nothing prevents using public inheritance in these ways,
but doing so leads to trouble.  Stick to "is-a".

Objects of the derived class can use the base class methods
without change.  You can also redefine methods for the
derived class. The way a method behaves may depend on the
object that invokes it. The idea of having multiple
behaviors for a method depending on the context is called
polymorphism.

Two key methods for implementing polymorphism for public
inheritance:

1. Redefining the base-class methods in a derived class.
2. Using virtual methods.

*/

#ifndef BRASS_H_
#define BRASS_H_
#include <string>
// Brass Account Class
class Brass {
private:
    std::string fullName;
    long acctNum;
protected:
    double balance;
public:
    Brass(const std::string & s = "Nobody", long an = -1,
          double bal = 0.0);
    double Balance() const;
    void Deposit(double amt);
    /* If tyou use "virtual", the program schooses a method
    based on the type of object the reference or pointer refers to.

    This is very handy!  It's common practice to define as virtual
    methods that might be redefined in a derived class.  Such a method
    is automatically virtual in the derived class, but it's good
    practice declare it as virtual for clarity, even if that's redundant.

    Note that the keyword virtual is used just in the method
    in the class declaration, not in the method definition.
    */
    virtual void Withdraw(double amt);
    virtual void ViewAcct() const;
    virtual ~Brass() {};
};

// Brass Plus Account Class
class BrassPlus : public Brass {
private:
    double maxLoan;
    double rate;
    double owesBank;
public:
    BrassPlus(const std::string & s = "Nullbody",
              long an = -1,
              double bal = 0.0,
              double ml = 500,
              double r = 0.11125);
    BrassPlus(const Brass & ba,
              double ml = 500,
              double r = 0.11125);
    virtual void ViewAcct() const;
    virtual void Withdraw(double amt);
    void ResetMax(double m) { maxLoan = m;}
    void ResetRate(double r) { rate = r;}
    void ResetOwes() {owesBank = 0; }
};
#endif // BRASS_H_
